@@ -149,6 +149,8 @@ gst_encoding_target_finalize (GObject * object)
 
   g_list_foreach (target->profiles, (GFunc) g_object_unref, NULL);
   g_list_free (target->profiles);
+
+  G_OBJECT_CLASS (gst_encoding_target_parent_class)->finalize (object);
 }
 
 static void
@@ -198,7 +200,7 @@ gst_encoding_target_get_description (GstEncodingTarget * target)
  * gst_encoding_target_get_path:
  * @target: a #GstEncodingTarget
  *
- * Returns: (transfer none): The path to the @target file.
+ * Returns: (transfer none) (nullable) (type filename): The path to the @target file.
  *
  * Since: 1.18
  */
@@ -227,7 +229,7 @@ gst_encoding_target_get_profiles (GstEncodingTarget * target)
  * @target: a #GstEncodingTarget
  * @name: the name of the profile to retrieve
  *
- * Returns: (transfer full): The matching #GstEncodingProfile, or %NULL.
+ * Returns: (nullable) (transfer full): The matching #GstEncodingProfile, or %NULL.
  */
 GstEncodingProfile *
 gst_encoding_target_get_profile (GstEncodingTarget * target, const gchar * name)
@@ -309,7 +311,7 @@ validate_name (const gchar * name)
  * *can* be a application or user specific category if
  * needed.
  *
- * Returns: (transfer full): The newly created #GstEncodingTarget or %NULL if
+ * Returns: (transfer full) (nullable): The newly created #GstEncodingTarget or %NULL if
  * there was an error.
  */
 
@@ -640,6 +642,8 @@ parse_encoding_profile (GKeyFile * in, gchar * parentprofilename,
   proftype = g_key_file_get_value (in, profilename, "type", NULL);
   if (!proftype) {
     GST_WARNING ("Missing 'type' field for streamprofile %s", profilename);
+    g_free (pname);
+    g_free (description);
     return NULL;
   }
 
@@ -898,7 +902,7 @@ gst_encoding_target_subload (gchar * path, const gchar * category,
  * @name: the name of the #GstEncodingTarget to load (automatically
  * converted to lower case internally as capital letters are not
  * valid for target names).
- * @category: (allow-none): the name of the target category, like
+ * @category: (nullable): the name of the target category, like
  * #GST_ENCODING_CATEGORY_DEVICE. Can be %NULL
  * @error: If an error occurred, this field will be filled in.
  *
@@ -1209,8 +1213,10 @@ sub_get_all_targets (gchar * subdir)
     if (target) {
       target->path = fullname;
       res = g_list_append (res, target);
-    } else
+    } else {
       GST_WARNING ("Failed to get a target from %s", fullname);
+      g_free (fullname);
+    }
   }
   g_dir_close (dir);
 
@@ -1280,7 +1286,7 @@ merge_targets (GList * res, GList * extra)
 
 /**
  * gst_encoding_list_all_targets:
- * @categoryname: (allow-none): The category, for ex: #GST_ENCODING_CATEGORY_DEVICE.
+ * @categoryname: (nullable): The category, for ex: #GST_ENCODING_CATEGORY_DEVICE.
  * Can be %NULL.
  *
  * List all available #GstEncodingTarget for the specified category, or all categories
