@@ -1181,6 +1181,7 @@ gst_audio_visualizer_chain (GstPad * pad, GstObject * parent,
       }
     }
 
+    gst_buffer_remove_all_memory (inbuf);
     gst_buffer_copy_into (inbuf, databuf, GST_BUFFER_COPY_MEMORY, 0, sbpf);
     gst_buffer_unref (databuf);
 
@@ -1263,7 +1264,7 @@ gst_audio_visualizer_src_event (GstPad * pad, GstObject * parent,
       if (diff >= 0)
         /* we're late, this is a good estimate for next displayable
          * frame (see part-qos.txt) */
-        scope->priv->earliest_time = timestamp + 2 * diff +
+        scope->priv->earliest_time = timestamp + MIN (2 * diff, GST_SECOND) +
             scope->priv->frame_duration;
       else
         scope->priv->earliest_time = timestamp + diff;
