@@ -21,16 +21,14 @@
 
 /**
  * SECTION:gsttagvorbis
+ * @title: GstVorbisTag
  * @short_description: tag mappings and support functions for plugins
  *                     dealing with vorbiscomments
  * @see_also: #GstTagList
  *
- * <refsect2>
- * <para>
  * Contains various utility functions for plugins to parse or create
  * vorbiscomments and map them to and from #GstTagList<!-- -->s.
- * </para>
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -75,10 +73,14 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_ALBUM_GAIN, "REPLAYGAIN_ALBUM_GAIN"},
   {GST_TAG_ALBUM_PEAK, "REPLAYGAIN_ALBUM_PEAK"},
   {GST_TAG_REFERENCE_LEVEL, "REPLAYGAIN_REFERENCE_LOUDNESS"},
+  {GST_TAG_ACOUSTID_ID, "ACOUSTID_ID"},
+  {GST_TAG_ACOUSTID_FINGERPRINT, "ACOUSTID_FINGERPRINT"},
   {GST_TAG_MUSICBRAINZ_TRACKID, "MUSICBRAINZ_TRACKID"},
+  {GST_TAG_MUSICBRAINZ_RELEASETRACKID, "MUSICBRAINZ_RELEASETRACKID"},
   {GST_TAG_MUSICBRAINZ_ARTISTID, "MUSICBRAINZ_ARTISTID"},
   {GST_TAG_MUSICBRAINZ_ALBUMID, "MUSICBRAINZ_ALBUMID"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "MUSICBRAINZ_ALBUMARTISTID"},
+  {GST_TAG_MUSICBRAINZ_RELEASEGROUPID, "MUSICBRAINZ_RELEASEGROUPID"},
   {GST_TAG_MUSICBRAINZ_TRMID, "MUSICBRAINZ_TRMID"},
   {GST_TAG_ARTIST_SORTNAME, "ARTISTSORT"},
   {GST_TAG_ARTIST_SORTNAME, "ARTISTSORTORDER"},
@@ -92,6 +94,7 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_ALBUM_ARTIST_SORTNAME, "ALBUMARTISTSORT"},
   {GST_TAG_ALBUM_ARTIST_SORTNAME, "ALBUMARTISTSORTORDER"},
   {GST_TAG_LANGUAGE_CODE, "LANGUAGE"},
+  {GST_TAG_LYRICS, "LYRICS"},
   {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "MUSICBRAINZ_DISCID"},
   {GST_TAG_CDDA_CDDB_DISCID, "DISCID"},
   /* For the apparent de-facto standard for coverart in vorbis comments, see:
@@ -115,7 +118,7 @@ static const GstTagEntryMatch tag_matches[] = {
  *
  * Looks up the GStreamer tag for a vorbiscomment tag.
  *
- * Returns: The corresponding GStreamer tag or NULL if none exists.
+ * Returns: (nullable): The corresponding GStreamer tag or NULL if none exists.
  */
 const gchar *
 gst_tag_from_vorbis_tag (const gchar * vorbis_tag)
@@ -144,7 +147,7 @@ gst_tag_from_vorbis_tag (const gchar * vorbis_tag)
  *
  * Looks up the vorbiscomment tag for a GStreamer tag.
  *
- * Returns: The corresponding vorbiscomment tag or NULL if none exists.
+ * Returns: (nullable): The corresponding vorbiscomment tag or NULL if none exists.
  */
 const gchar *
 gst_tag_to_vorbis_tag (const gchar * gst_tag)
@@ -405,17 +408,17 @@ decode_failed:
 
 /**
  * gst_tag_list_from_vorbiscomment:
- * @data: data to convert
+ * @data: (array length=size): data to convert
  * @size: size of @data
- * @id_data: identification data at start of stream
+ * @id_data: (array length=id_data_length): identification data at start of stream
  * @id_data_length: length of identification data
- * @vendor_string: pointer to a string that should take the vendor string
- *                 of this vorbis comment or NULL if you don't need it.
+ * @vendor_string: (out) (optional): pointer to a string that should take the
+ *     vendor string of this vorbis comment or NULL if you don't need it.
  *
  * Creates a new tag list that contains the information parsed out of a
  * vorbiscomment packet.
  *
- * Returns: A new #GstTagList with all tags that could be extracted from the
+ * Returns: (transfer full) (nullable): A new #GstTagList with all tags that could be extracted from the
  *          given vorbiscomment buffer or NULL on error.
  */
 GstTagList *
@@ -482,7 +485,7 @@ gst_tag_list_from_vorbiscomment (const guint8 * data, gsize size,
       gst_vorbis_tag_add_coverart (list, value, value_len);
     } else if (g_ascii_strcasecmp (cur, "METADATA_BLOCK_PICTURE") == 0) {
       gst_vorbis_tag_add_metadata_block_picture (list, value, value_len);
-    } else {
+    } else if (g_utf8_validate (cur, -1, NULL)) {
       gst_vorbis_tag_add (list, cur, value);
     }
     g_free (cur);
@@ -491,6 +494,10 @@ gst_tag_list_from_vorbiscomment (const guint8 * data, gsize size,
   return list;
 
 error:
+  if (vendor_string && *vendor_string) {
+    g_free (*vendor_string);
+    *vendor_string = NULL;
+  }
   gst_tag_list_unref (list);
   return NULL;
 #undef ADVANCE
@@ -499,15 +506,15 @@ error:
 /**
  * gst_tag_list_from_vorbiscomment_buffer:
  * @buffer: buffer to convert
- * @id_data: identification data at start of stream
+ * @id_data: (array length=id_data_length): identification data at start of stream
  * @id_data_length: length of identification data
- * @vendor_string: pointer to a string that should take the vendor string
- *                 of this vorbis comment or NULL if you don't need it.
+ * @vendor_string: (out) (optional): pointer to a string that should take the
+ *     vendor string of this vorbis comment or NULL if you don't need it.
  *
  * Creates a new tag list that contains the information parsed out of a
  * vorbiscomment packet.
  *
- * Returns: A new #GstTagList with all tags that could be extracted from the
+ * Returns: (transfer full) (nullable): A new #GstTagList with all tags that could be extracted from the
  *          given vorbiscomment buffer or NULL on error.
  */
 GstTagList *
@@ -568,10 +575,23 @@ gst_tag_to_metadata_block_picture (const gchar * tag,
     mime_type = "-->";
   mime_type_len = strlen (mime_type);
 
+  /* FIXME 2.0: Remove the image-type reading from the caps, this was
+   * a bug until 1.2.2. The image-type is only supposed to be in the
+   * info structure */
   gst_structure_get (mime_struct, "image-type", GST_TYPE_TAG_IMAGE_TYPE,
       &image_type, "width", G_TYPE_INT, &width, "height", G_TYPE_INT, &height,
       NULL);
 
+  if (image_type == GST_TAG_IMAGE_TYPE_NONE) {
+    const GstStructure *info_struct;
+
+    info_struct = gst_sample_get_info (sample);
+    if (info_struct && gst_structure_has_name (info_struct, "GstTagImageInfo")) {
+      gst_structure_get (info_struct, "image-type", GST_TYPE_TAG_IMAGE_TYPE,
+          &image_type, NULL);
+    }
+  }
+
   metadata_block_len = 32 + mime_type_len + gst_buffer_get_size (buffer);
   gst_byte_writer_init_with_size (&writer, metadata_block_len, TRUE);
 
@@ -762,13 +782,13 @@ write_one_tag (const GstTagList * list, const gchar * tag, gpointer user_data)
 /**
  * gst_tag_list_to_vorbiscomment_buffer:
  * @list: tag list to convert
- * @id_data: identification data at start of stream
+ * @id_data: (array length=id_data_length): identification data at start of stream
  * @id_data_length: length of identification data, may be 0 if @id_data is NULL
- * @vendor_string: string that describes the vendor string or NULL
+ * @vendor_string: (nullable): string that describes the vendor string or NULL
  *
  * Creates a new vorbiscomment buffer from a tag list.
  *
- * Returns: A new #GstBuffer containing a vorbiscomment buffer with all tags
+ * Returns: (transfer full): A new #GstBuffer containing a vorbiscomment buffer with all tags
  *          that could be converted from the given tag list.
  */
 GstBuffer *
