@@ -377,42 +377,6 @@ extern "C" {
  */
 #define DRM_FORMAT_Q401		fourcc_code('Q', '4', '0', '1')
 
-/*
- * 3 plane YCbCr LSB aligned
- * In order to use these formats in a similar fashion to MSB aligned ones
- * implementation can multiply the values by 2^6=64. For that reason the padding
- * must only contain zeros.
- * index 0 = Y plane, [15:0] z:Y [6:10] little endian
- * index 1 = Cr plane, [15:0] z:Cr [6:10] little endian
- * index 2 = Cb plane, [15:0] z:Cb [6:10] little endian
- */
-#define DRM_FORMAT_S010	fourcc_code('S', '0', '1', '0') /* 2x2 subsampled Cb (1) and Cr (2) planes 10 bits per channel */
-#define DRM_FORMAT_S210	fourcc_code('S', '2', '1', '0') /* 2x1 subsampled Cb (1) and Cr (2) planes 10 bits per channel */
-#define DRM_FORMAT_S410	fourcc_code('S', '4', '1', '0') /* non-subsampled Cb (1) and Cr (2) planes 10 bits per channel */
-
-/*
- * 3 plane YCbCr LSB aligned
- * In order to use these formats in a similar fashion to MSB aligned ones
- * implementation can multiply the values by 2^4=16. For that reason the padding
- * must only contain zeros.
- * index 0 = Y plane, [15:0] z:Y [4:12] little endian
- * index 1 = Cr plane, [15:0] z:Cr [4:12] little endian
- * index 2 = Cb plane, [15:0] z:Cb [4:12] little endian
- */
-#define DRM_FORMAT_S012	fourcc_code('S', '0', '1', '2') /* 2x2 subsampled Cb (1) and Cr (2) planes 12 bits per channel */
-#define DRM_FORMAT_S212	fourcc_code('S', '2', '1', '2') /* 2x1 subsampled Cb (1) and Cr (2) planes 12 bits per channel */
-#define DRM_FORMAT_S412	fourcc_code('S', '4', '1', '2') /* non-subsampled Cb (1) and Cr (2) planes 12 bits per channel */
-
-/*
- * 3 plane YCbCr
- * index 0 = Y plane, [15:0] Y little endian
- * index 1 = Cr plane, [15:0] Cr little endian
- * index 2 = Cb plane, [15:0] Cb little endian
- */
-#define DRM_FORMAT_S016	fourcc_code('S', '0', '1', '6') /* 2x2 subsampled Cb (1) and Cr (2) planes 16 bits per channel */
-#define DRM_FORMAT_S216	fourcc_code('S', '2', '1', '6') /* 2x1 subsampled Cb (1) and Cr (2) planes 16 bits per channel */
-#define DRM_FORMAT_S416	fourcc_code('S', '4', '1', '6') /* non-subsampled Cb (1) and Cr (2) planes 16 bits per channel */
-
 /*
  * 3 plane YCbCr
  * index 0: Y plane, [7:0] Y
@@ -458,7 +422,6 @@ extern "C" {
 #define DRM_FORMAT_MOD_VENDOR_ALLWINNER 0x09
 #define DRM_FORMAT_MOD_VENDOR_AMLOGIC 0x0a
 #define DRM_FORMAT_MOD_VENDOR_MTK     0x0b
-#define DRM_FORMAT_MOD_VENDOR_APPLE   0x0c
 
 /* add more to the end as needed */
 
@@ -1531,50 +1494,6 @@ drm_fourcc_canonicalize_nvidia_format_mod(guint64 modifier)
 /* alias for the most common tiling format */
 #define DRM_FORMAT_MOD_MTK_16L_32S_TILE  DRM_FORMAT_MOD_MTK(MTK_FMT_MOD_TILE_16L32S)
 
-/*
- * Apple GPU-tiled layouts.
- *
- * Apple GPUs support nonlinear tilings with optional lossless compression.
- *
- * GPU-tiled images are divided into 16KiB tiles:
- *
- *     Bytes per pixel  Tile size
- *     ---------------  ---------
- *                   1  128x128
- *                   2  128x64
- *                   4  64x64
- *                   8  64x32
- *                  16  32x32
- *
- * Tiles are raster-order. Pixels within a tile are interleaved (Morton order).
- *
- * Compressed images pad the body to 128-bytes and are immediately followed by a
- * metadata section. The metadata section rounds the image dimensions to
- * powers-of-two and contains 8 bytes for each 16x16 compression subtile.
- * Subtiles are interleaved (Morton order).
- *
- * All images are 128-byte aligned.
- *
- * These layouts fundamentally do not have meaningful strides. No matter how we
- * specify strides for these layouts, userspace unaware of Apple image layouts
- * will be unable to use correctly the specified stride for any purpose.
- * Userspace aware of the image layouts do not use strides. The most "correct"
- * convention would be setting the image stride to 0. Unfortunately, some
- * software assumes the stride is at least (width * bytes per pixel). We
- * therefore require that stride equals (width * bytes per pixel). Since the
- * stride is arbitrary here, we pick the simplest convention.
- *
- * Although containing two sections, compressed image layouts are treated in
- * software as a single plane. This is modelled after AFBC, a similar
- * scheme. Attempting to separate the sections to be "explicit" in DRM would
- * only generate more confusion, as software does not treat the image this way.
- *
- * For detailed information on the hardware image layouts, see
- * https://docs.mesa3d.org/drivers/asahi.html#image-layouts
- */
-#define DRM_FORMAT_MOD_APPLE_GPU_TILED fourcc_mod_code(APPLE, 1)
-#define DRM_FORMAT_MOD_APPLE_GPU_TILED_COMPRESSED fourcc_mod_code(APPLE, 2)
-
 /*
  * AMD modifiers
  *
