@@ -64,7 +64,7 @@ add_source (gdouble freq)
   info->srcpad = gst_element_get_static_pad (info->element, "src");
 
   /* get new pad from adder, adder will now wait for data on this pad */
-  info->sinkpad = gst_element_get_request_pad (adder, "sink_%u");
+  info->sinkpad = gst_element_request_pad_simple (adder, "sink_%u");
 
   /* link pad to adder */
   gst_pad_link (info->srcpad, info->sinkpad);
